 
--localization
local _, L = ...

--define global vars and defaults (saved between sessions)
local function SetDefaults()
	KLM_Settings = {
		["DoHealth"] = true,
		["DoPower"] = true,
		["SpamLimiter"] = 120,
		["AudibleAlertsOn"] = true,
		["FlashAlertsOn"] = true,
		["HUDOn"] = true,
		["AnnouncePartyOn"] = false,
		["AnnounceRaidOn"] = false,
		["HealthLow"] = 30,
		["HealthLowLow"] = 15,
		["PowerLow"] = 30,
		["PowerLowLow"] = 15,
	}
end
SetDefaults()

--replace print function
local function print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(L["KLM"]..": " .. tostring(msg))
end

--local vars
local debugHealth = nil
local debugPower = nil
--define static HEALTH vars and initialize them
local LastHealthMax = 0
local LastHealthLevel = 0
--local LastHealthPercent = HealthPercent
local HealthAlarmLevel = 4  -- 1=LowLow, 2=Low, 3=Normal, 4=Full
local FlashHealth = false
--health chatbox SPAMMING control
local LastHealthAnnounce = "" --what was the last chat msg for health?
local HealthSpamTimer = 0
--define static POWER vars and initialize them
local PowerTypeEnabled = false --do we want to monitor the current power type or disable?
local LastPowerType = 0
local LastPowerTypeS = ""
local LastPowerMax = 0
local LastPowerLevel = 0
local LastPowerPercent = 0
local LastPowerTypeEnabled = PowerTypeEnabled
local PowerAlarmLevel = 4  -- 0=Empty, 1=LowLow, 2=Low, 3=Normal, 4=Full
local FlashPower = false
--power chatbox SPAMMING control
local LastPowerAnnounce = "" --what was the last chat msg for power?
local PowerSpamTimer = 0
-- miscellanious
local SpecChanged = false
local ElapsedTotal = 0
local UpdateInterval = 0.1
local UpdateCounter = 0
local Arghed = false
local DelayAlert = nil
local DelayTime = 3

local function Update()
	local MTHealth={} --"Message Table" for health to build all the msgs to display at end of loop
	local MTPower={} --"Message Table" for power to build all the msgs to display at end of loop
	local DoAlarmOnce=false
	local Role = UnitGroupRolesAssigned("player")
	local Index

	if KLM_Settings.DoHealth then
		--gather new health values
		local HealthMax = UnitHealthMax("player")
		if not HealthMax or HealthMax == 0 then return; end
		local HealthLevel = UnitHealth("player")
		if not HealthLevel then return; end
		local HealthPercent = floor(100 * HealthLevel / HealthMax) + 1
		if (debugHealth ~= nil) then HealthPercent = debugHealth; end

		--notify on health level changes
		if HealthMax ~= LastHealthMax then
			table.insert(MTHealth,{Msg=L["Max health changed."],HUD=false,Alarm=false,TTL=1,Announce=false,Print=false})
		end
		--if there is a health level change then build messages
		if ((HealthPercent ~= LastHealthPercent) or ((FlashHealth == true) and (HealthPercent == 100))) then
			--health
			if HealthPercent <= KLM_Settings.HealthLowLow then
				table.insert(MTHealth,{Msg=L["Almost dead!"].." ("..HealthPercent.."%)",HUD=false,Alarm=false,TTL=1,Announce=false,Print=false})
				FlashHealth = true --show flash
				MTHealth[#MTHealth].HUD=true --repeatedly show on HUD
				if HealthAlarmLevel > 1 then --only do once when changing to this alarm level
					MTHealth[#MTHealth].Alarm=true
				end
				HealthAlarmLevel = 1
			elseif HealthPercent <= KLM_Settings.HealthLow then
				table.insert(MTHealth,{Msg=L["Health is low."].." ("..HealthPercent.."%)",HUD=false,Alarm=false,TTL=1,Announce=false,Print=false})
				FlashHealth = false --hide flash
				if HealthAlarmLevel > 2 then --only do once when changing to this alarm level
					MTHealth[#MTHealth].HUD=true
					MTHealth[#MTHealth].Alarm=true
				end
				HealthAlarmLevel = 2
			elseif HealthPercent > KLM_Settings.HealthLow and HealthPercent < 95 then
				FlashHealth = false --hide flash
				if HealthAlarmLevel > 3 then --only do once when changing to this alarm level
					--nothing to do at this time
				end
				HealthAlarmLevel = 3
			elseif HealthPercent == 100 then 
				FlashHealth = false --hide flash
				if HealthAlarmLevel < 4 then --only do once when changing to this alarm level
					table.insert(MTHealth,{Msg=L["Full health restored."],HUD=true,Alarm=false,TTL=1,Announce=false,Print=false})
				end
				HealthAlarmLevel = 4
			end
		elseif UnitIsDeadOrGhost("player") then
			-- if not Arghed then 
				-- table.insert(MTHealth,{Msg=L["Argh!"],HUD=true,Alarm=false,TTL=1,Announce=false,Print=false})
				-- Arghed = true
			-- end
			FlashHealth = false --hide flash
		-- elseif not UnitIsDeadOrGhost("player") then
			-- Arghed = false
		end

		--save old health values
		LastHealthMax = HealthMax
		LastHealthLevel = HealthLevel
		LastHealthPercent = HealthPercent
		
		--display any health alerts
		for Index=1, #MTHealth, 1 do 
			--display any health alerts
			KLMFrame:SetTimeVisible(MTHealth[Index].TTL)  --how long to show in HUD
			if MTHealth[Index].HUD and KLM_Settings.HUDOn then 
				KLMFrame:AddMessage(MTHealth[Index].Msg, 0, 1, 0, nil, 3) --show in HUD
			end
			if MTHealth[Index].Announce and Role == "HEALER" and not UnitIsDeadOrGhost("player") then  --announce to chatbox?
				if UnitInRaid("player") and KLM_Settings.AnnounceRaidOn then
					SendChatMessage(upper(L[string.lower(Role)])..": "..MTHealth[Index].Msg, "RAID")
				elseif not UnitInRaid("player") and GetNumSubgroupMembers() > 0 and KLM_Settings.AnnouncePartyOn then
					SendChatMessage(upper(L[string.lower(Role)])..": "..MTHealth[Index].Msg, "PARTY")
				else
					print(upper(L[string.lower(Role)])..": "..MTHealth[Index].Msg)
				end
			end
			if MTHealth[Index].Alarm then DoAlarmOnce=true; end
		end
	end

	if KLM_Settings.DoPower then
		--gather new power values
		local PowerType, PowerTypeS = UnitPowerType("player")
		if not PowerType or not PowerTypeS then return; end
		local PowerMax = UnitPowerMax("player")
		if not PowerMax or PowerMax == 0 then return; end
		local PowerLevel = UnitPower("player")
		if not PowerLevel then return; end
		PowerPercent = floor(100 * PowerLevel / PowerMax) + 1
		if (debugPower ~= nil) then PowerPercent = debugPower; end
		--notify on power type and power maximum changes
		--do we want to monitor the current powertype?
		if PowerType == SPELL_POWER_MANA then PowerTypeEnabled = true --0 to variable
		elseif PowerType == SPELL_POWER_RAGE then PowerTypeEnabled = false --0 to 100
		elseif PowerType == SPELL_POWER_FOCUS then PowerTypeEnabled = false --100 to 0
		elseif PowerType == SPELL_POWER_ENERGY then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_COMBO_POINTS then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_RUNES then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_RUNIC_POWER then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_SOUL_SHARDS then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_LUNAR_POWER then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_HOLY_POWER then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_ALTERNATE_POWER then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_MAELSTROM then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_CHI then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_INSANITY then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_OBSOLETE then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_OBSOLETE2 then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_ARCANE_CHARGES then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_FURY then PowerTypeEnabled = false --??
		elseif PowerType == SPELL_POWER_PAIN then PowerTypeEnabled = false --??
		else PowerTypeEnabled = false --??
		end
		if PowerType ~= LastPowerType then
			table.insert(MTPower,{Msg=PowerTypeS.." "..L["has replaced"].." "..LastPowerTypeS..".",HUD=false,Alarm=false,TTL=6,Announce=false,Print=true})
		elseif (PowerMax ~= LastPowerMax) and PowerTypeEnabled then
			table.insert(MTPower,{Msg=L["Max"].." "..PowerTypeS.." "..L["changed."],HUD=false,Alarm=false,TTL=1,Announce=false,Print=false})
		end
		if PowerTypeEnabled ~= LastPowerTypeEnabled then
			if PowerTypeEnabled then
				table.insert(MTPower,{Msg=L["Enabling power monitoring for"].." "..PowerTypeS.." "..L["power type."],HUD=false,Alarm=false,TTL=6,Announce=false,Print=true})
			else
				table.insert(MTPower,{Msg=L["Disabling power monitoring for"].." "..PowerTypeS.." "..L["power type."],HUD=false,Alarm=false,TTL=6,Announce=false,Print=true})
			end
		end
		--if there is a power level change then build messages
		if (PowerPercent ~= LastPowerPercent) and PowerTypeEnabled then
			--power
			if PowerPercent == 0 then
				table.insert(MTPower,{Msg=L["No"].." "..PowerTypeS.." "..L["remaining."],HUD=false,Alarm=false,TTL=1,Announce=false,Print=false})
				MTPower[#MTPower].HUD=true --repeatedly show on HUD
				FlashPower = false --hide flash
				if PowerAlarmLevel > 0 then --only do once when changing to this alarm level
					MTPower[#MTPower].Alarm=true
					if (LastPowerAnnounce ~= MTPower[#MTPower].Msg) and (time() - PowerSpamTimer > KLM_Settings.SpamLimiter) then
						MTPower[#MTPower].Announce=true; PowerSpamTimer = time() --enable chatbox announcement and reset spam timer
						LastPowerAnnounce=MTPower[#MTPower].Msg
					end
				end
				PowerAlarmLevel = 0
			elseif PowerPercent <= KLM_Settings.PowerLowLow then
				table.insert(MTPower,{Msg=L["Almost out of"].." "..PowerTypeS.." ("..PowerPercent.."%)",HUD=false,Alarm=false,TTL=1,Announce=false,Print=false})
				MTPower[#MTPower].HUD=true --repeatedly show on HUD
				FlashPower = true --show flash
				if PowerAlarmLevel > 1 then --only do once when changing to this alarm level
					MTPower[#MTPower].Alarm=true
					if (LastPowerAnnounce ~= MTPower[#MTPower].Msg) and (time() - PowerSpamTimer > KLM_Settings.SpamLimiter) then
						MTPower[#MTPower].Announce=true; PowerSpamTimer = time() --enable chatbox announcement and reset spam timer
						LastPowerAnnounce=MTPower[#MTPower].Msg
					end
				end
				PowerAlarmLevel = 1
			elseif PowerPercent <= KLM_Settings.PowerLow then
				table.insert(MTPower,{Msg=PowerTypeS.." "..L["is low."].." ("..PowerPercent.."%)",HUD=false,Alarm=false,TTL=1,Announce=false,Print=false})
				FlashPower = false --hide flash
				if PowerAlarmLevel > 2 then --only do once when changing to this alarm level
					MTPower[#MTPower].HUD=true
					MTPower[#MTPower].Alarm=true
				end
				PowerAlarmLevel = 2
			elseif PowerPercent > KLM_Settings.PowerLow and PowerPercent < 95 then 
				FlashPower = false --hide flash
				if PowerAlarmLevel > 3 then --only do once when changing to this alarm level
					--nothing to do at this time
				end
				PowerAlarmLevel = 3
			elseif PowerPercent == 100 then 
				FlashPower = false --hide flash
				if PowerAlarmLevel < 4 then --only do once when changing to this alarm level
					table.insert(MTPower,{Msg=L["Full"].." "..PowerTypeS.." "..L["restored."],HUD=true,Alarm=false,TTL=1,Announce=false,Print=false})
				end
				PowerAlarmLevel = 4 --reset audible alarm
			end
			if PowerPercent > KLM_Settings.PowerLowLow then
				SpecChanged = false --make sure we are announcing to chat now
			end
		end
		
		--save old power values
		LastPowerType = PowerType
		LastPowerTypeS = PowerTypeS
		LastPowerMax = PowerMax
		LastPowerLevel = PowerLevel
		LastPowerPercent = PowerPercent
		LastPowerTypeEnabled = PowerTypeEnabled

		--display any power alerts
		for Index=1, #MTPower, 1 do 
			--display any power alerts
			KLMFrame:SetTimeVisible(MTPower[Index].TTL)  --how long to show in HUD
			if MTPower[Index].HUD and KLM_Settings.HUDOn then
				KLMFrame:AddMessage(MTPower[Index].Msg, PowerBarColor[PowerTypeS].r, PowerBarColor[PowerTypeS].g, PowerBarColor[PowerTypeS].b, nil, 3) --show in HUD
			end
			if MTPower[Index].Announce and Role == "HEALER" and not UnitIsDeadOrGhost("player") then  --announce to chatbox?
				if UnitInRaid("player") and KLM_Settings.AnnounceRaidOn and SpecChanged == false then
					SendChatMessage(upper(L[string.lower(Role)])..": "..MTPower[Index].Msg, "RAID")
				elseif not UnitInRaid("player") and GetNumSubgroupMembers() > 0 and KLM_Settings.AnnouncePartyOn and SpecChanged == false then
					SendChatMessage(upper(L[string.lower(Role)])..": "..MTPower[Index].Msg, "PARTY")
				else
					print(upper(L[string.lower(Role)])..": "..MTPower[Index].Msg)
				end
			end
			if MTPower[Index].Alarm then DoAlarmOnce=true; end
		end
	end
	
	--play alarm but only once for this loop of messages
	if DoAlarmOnce and KLM_Settings.AudibleAlertsOn then
		PlaySoundFile("Interface\\addons\\KLifeMon\\HyprBlip.mp3")
	end
	--flash screen alert
	if KLM_Settings.FlashAlertsOn then
		if UnitIsDeadOrGhost("player") then
			KLMFlasher() --hide flash
		elseif FlashHealth == true then
			KLMFlasher("HEALTH") --show flash
		elseif FlashPower == true then
			KLMFlasher(PowerTypeS) --show flash
		else
			KLMFlasher() --hide flash
		end
	end
	
	if ((debugHealth) and (debugHealth > 0) and (debugHealth < 100)) then 
		debugHealth = debugHealth - 1
		print("debugHealth="..debugHealth)
	end
	if ((debugPower) and (debugPower > 0) and (debugPower < 100)) then 
		debugPower = debugPower - 1
		print("debugPower="..debugPower)
	end
end

function KLMFlasher(Type)
	local Image = "None"
	if(Type == "MANA") then
		Image = "Interface\\FullScreenTextures\\OutofControl"
	elseif(Type == "HEALTH") then
		Image = "Interface\\FullScreenTextures\\LowHealth"
	else
		Image = nil
	end

	if Image then
		KLMFlashFrame_Texture:SetTexture(Image)
		KLMFlashFrame_Texture:SetBlendMode("ADD")
		KLMFlashFrame_Texture:SetAllPoints("UIParent")
		KLMFlashFrame_Texture:Show()
	else
		KLMFlashFrame_Texture:Hide()
	end
end

function KLMFlashFrame_OnShow(self)
	self:SetAlpha(0)
end

function KLMFlashFrame_OnUpdate(self, elapsed)
	ElapsedTotal = ElapsedTotal + elapsed
	UpdateCounter = UpdateCounter + elapsed
	if UpdateCounter >= UpdateInterval then
		UpdateCounter = 0
		Alpha = (abs(1 - (ElapsedTotal % 2)) * 10) / 10
		self:SetAlpha(Alpha)
	end
end

function KLMFrame_OnLoad(self)
    KLMFrame:RegisterEvent("UNIT_POWER_UPDATE")
    KLMFrame:RegisterEvent("UNIT_MAXPOWER")
    KLMFrame:RegisterEvent("UNIT_HEALTH")
    KLMFrame:RegisterEvent("UNIT_MAXHEALTH")
    KLMFrame:RegisterEvent("UNIT_DISPLAYPOWER")
    KLMFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
    KLMFrame:RegisterEvent("RAID_ROSTER_UPDATE")
    KLMFrame:RegisterEvent("ROLE_CHANGED_INFORM")
    KLMFrame:RegisterEvent("ADDON_LOADED")
	KLMFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	KLMFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	KLMFrame:RegisterEvent("ZONE_CHANGED")
	KLMFrame:SetWidth(450)
	KLMFrame:SetHeight(200)
	KLMFrame:SetPoint("CENTER",UIParent,"CENTER",0,0)	
	KLMFrame:SetFont("Fonts\\ARIALN.TTF", 30, "THICKOUTLINE")
	KLMFrame:SetShadowColor(0.00,0.00,0.00,0.75)
	KLMFrame:SetShadowOffset(3.00,-3.00)
	KLMFrame:SetJustifyH("CENTER")		
	KLMFrame:SetMaxLines(2)
	KLMFrame:SetTimeVisible(1)
	KLMFrame:SetFadeDuration(1)
	print(string.format(L["Life Monitor loaded.  Use /KLM or /%s for the options. (translators are needed)"], L["LifeMonitor"]))
end

function KLMFrame_OnEvent(Frame, Event, ...)
	if Event == "ADDON_LOADED" and select(1,...) == "KLifeMon" then
		--vars loaded from saved settings
		LoadOptionSettings() --load global vars to option controls
	elseif Event == "ACTIVE_TALENT_GROUP_CHANGED" then
		if not (debugHealth and debugPower) then
			DelayAlert = time()
		end
		SpecChanged = true --make sure we are not announcing to chat for spec changes
	elseif Event == "ZONE_CHANGED_NEW_AREA" then
		--sometimes the flasher doesnt clear after leaving BG's etc so lets automatically clear it and
		--it can retrip if needed
		--KLMFlasher() --hide flash
		if not (debugHealth and debugPower) then
			DelayAlert = time()
		end
	elseif ((Event == "ZONE_CHANGED") and (debugHealth or debugPower)) then
		Update()
	elseif Event == "UNIT_MAXPOWER" or Event == "UNIT_MAXHEALTH" then
		DelayAlert = time()
	else
		if DelayAlert then
			if (time() - DelayAlert) >= DelayTime then
				DelayAlert = nil
				Update()
			end
		else
			Update()
		end
	end
end

function KLMOptionsFrame_OnLoad(panel)
	local WoWver = select(4, GetBuildInfo())
	-- Set the name for the Category for the Panel
	panel.name = L["Life Monitor"]--.." " .. GetAddOnMetadata("KLifeMon", "Version")

	--define the functions that will run when the user clicks on the associated buttons
	panel.okay = function(self) KLMOptions_Close(); end;
	panel.cancel = function(self)  KLMOptions_Cancel(); end;
	panel.default = function(self)  KLMOptions_Defaults(); end;
	--panel.parent = ""
	--panel.refresh = function(self)  KLMOptions_Refresh();  end;

	-- Add the panel to the Interface Options
	InterfaceOptions_AddCategory(panel)

	--define the direct to options command
	SlashCmdList["KLMOptions"] = KLMOptions_Command
	SLASH_KLMOptions1,SLASH_KLMOptions2,SLASH_KLMOptions3 = "/klm","/"..L["LifeMonitor"],"/LifeMon"

	--setup options frame controls
	KLMOptionsFrame_Title:SetText(L["Life Monitor"]..' r40')
	
	KLMOptionsFrame_CBHealth_Text:SetText(L["Monitor health (HP)"])
	
	KLMOptionsFrame_HealthLowSliderText:SetText(L["Health low %"])
	KLMOptionsFrame_HealthLowSliderHigh:SetText("100")
	KLMOptionsFrame_HealthLowSliderLow:SetText("1")
	KLMOptionsFrame_HealthLowSlider:SetMinMaxValues(1, 100)
	KLMOptionsFrame_HealthLowSlider:SetValueStep(1)

	KLMOptionsFrame_HealthLowLowSliderText:SetText(L['Health very low %'])
	KLMOptionsFrame_HealthLowLowSliderHigh:SetText("100")
	KLMOptionsFrame_HealthLowLowSliderLow:SetText("1")
	KLMOptionsFrame_HealthLowLowSlider:SetMinMaxValues(1, 100)
	KLMOptionsFrame_HealthLowLowSlider:SetValueStep(1)
	
	KLMOptionsFrame_CBPower_Text:SetText(L['Monitor mana'])
	
	KLMOptionsFrame_PowerLowSliderText:SetText(L['Power low %'])
	KLMOptionsFrame_PowerLowSliderHigh:SetText("100")
	KLMOptionsFrame_PowerLowSliderLow:SetText("1")
	KLMOptionsFrame_PowerLowSlider:SetMinMaxValues(1, 100)
	KLMOptionsFrame_PowerLowSlider:SetValueStep(1)

	KLMOptionsFrame_PowerLowLowSliderText:SetText(L['Power very low %'])
	KLMOptionsFrame_PowerLowLowSliderHigh:SetText("100")
	KLMOptionsFrame_PowerLowLowSliderLow:SetText("1")
	KLMOptionsFrame_PowerLowLowSlider:SetMinMaxValues(1, 100)
	KLMOptionsFrame_PowerLowLowSlider:SetValueStep(1)
						
	KLMOptionsFrame_CBAudible_Text:SetText(L['Enable audible alerts'])
	KLMOptionsFrame_CBFlash_Text:SetText(L['Enable screen flash alerts (very low alerts)'])
	KLMOptionsFrame_CBHUD_Text:SetText(L['Enable HUD alerts'])
	KLMOptionsFrame_CBParty_Text:SetText(L['Enable party alerts (Healers)'])
	KLMOptionsFrame_CBRaid_Text:SetText(L['Enable raid alerts (Healers)'])
	
	KLMOptionsFrame_SpamLimiterSliderText:SetText(L['Spam limiter'])
	KLMOptionsFrame_SpamLimiterSliderHigh:SetText('300 (5 '..L['mins']..')')
	KLMOptionsFrame_SpamLimiterSliderLow:SetText('1')
	KLMOptionsFrame_SpamLimiterSlider:SetMinMaxValues(1, 300)
	KLMOptionsFrame_SpamLimiterSlider:SetValueStep(1)
end

function KLMOptions_Command(Msg, Editbox)
	if (string.match(Msg, '^(.*)$') ~= '') then
		Msg = string.lower(Msg)
		if (string.match(Msg, '^(health%s%d+)$')) then
			debugHealth = tonumber(string.match(Msg, '(%d+)'))
			Update()
		elseif (string.match(Msg, '^(power%s%d+)$')) then
			debugPower = tonumber(string.match(Msg, '(%d+)'))
			Update()
		elseif (string.match(Msg, '^(reset)$')) then
			debugHealth = 100
			debugPower = 100
			Update()
		end
	else
		InterfaceOptionsFrame_OpenToCategory(KLMOptionsFrame)
		InterfaceOptionsFrame_OpenToCategory(KLMOptionsFrame)
		InterfaceOptionsFrame_OpenToCategory(KLMOptionsFrame)
		InterfaceOptionsFrame_OpenToCategory(KLMOptionsFrame)
	end
end

function KLMOptions_Close()
	--save option control settings to global vars
	KLM_Settings.DoHealth=KLMOptionsFrame_CBHealth:GetChecked()
	KLM_Settings.HealthLow=tonumber(KLMOptionsFrame_HealthLowSlider:GetValue())
	KLM_Settings.HealthLowLow=tonumber(KLMOptionsFrame_HealthLowLowSlider:GetValue())
	KLM_Settings.DoPower=KLMOptionsFrame_CBPower:GetChecked()
	KLM_Settings.PowerLow=tonumber(KLMOptionsFrame_PowerLowSlider:GetValue())
	KLM_Settings.PowerLowLow=tonumber(KLMOptionsFrame_PowerLowLowSlider:GetValue())
	KLM_Settings.SpamLimiter=KLMOptionsFrame_SpamLimiterSlider:GetValue()
	KLM_Settings.AudibleAlertsOn=KLMOptionsFrame_CBAudible:GetChecked()
	KLM_Settings.FlashAlertsOn=KLMOptionsFrame_CBFlash:GetChecked()
	if not KLM_Settings.FlashAlertsOn then KLMFlasher(); end --make sure not actively flashing if disabled.
	KLM_Settings.HUDOn=KLMOptionsFrame_CBHUD:GetChecked()
	KLM_Settings.AnnouncePartyOn=KLMOptionsFrame_CBParty:GetChecked()
	KLM_Settings.AnnounceRaidOn=KLMOptionsFrame_CBRaid:GetChecked()
end

function KLMOptions_Cancel()
	LoadOptionSettings()
end

function KLMOptions_Defaults()
	SetDefaults()
	LoadOptionSettings()
end

function LoadOptionSettings()
	--set option controls from global vars
	KLMOptionsFrame_CBHealth:SetChecked(KLM_Settings.DoHealth)
	KLMOptionsFrame_HealthLowSlider:SetValue(KLM_Settings.HealthLow)
	KLMOptionsFrame_HealthLowLowSlider:SetValue(KLM_Settings.HealthLowLow)
	KLMOptionsFrame_CBPower:SetChecked(KLM_Settings.DoPower)
	KLMOptionsFrame_PowerLowSlider:SetValue(KLM_Settings.PowerLow)
	KLMOptionsFrame_PowerLowLowSlider:SetValue(KLM_Settings.PowerLowLow)
	KLMOptionsFrame_SpamLimiterSlider:SetValue(KLM_Settings.SpamLimiter)
	KLMOptionsFrame_CBAudible:SetChecked(KLM_Settings.AudibleAlertsOn)
	KLMOptionsFrame_CBFlash:SetChecked(KLM_Settings.FlashAlertsOn)
	KLMOptionsFrame_CBHUD:SetChecked(KLM_Settings.HUDOn)
	KLMOptionsFrame_CBParty:SetChecked(KLM_Settings.AnnouncePartyOn)
	KLMOptionsFrame_CBRaid:SetChecked(KLM_Settings.AnnounceRaidOn)
end

function KLMOptionsFrame_SpamLimiterSlider_OnValueChanged(self, value)
	KLMOptionsFrame_SpamLimiterSlider_Seconds:SetText(tostring(floor(value)).." "..L["second(s)"])
end

function KLMOptionsFrame_HealthLowSlider_OnValueChanged(self, value)
	if value <= tonumber(KLMOptionsFrame_HealthLowLowSlider:GetValue()) then
		value = tonumber(KLMOptionsFrame_HealthLowLowSlider:GetValue()) + 1
		KLMOptionsFrame_HealthLowSlider:SetValue(value)
	end
	KLMOptionsFrame_HealthLowSlider_Percent:SetText(tostring(floor(value)).." "..L["percent"])
end

function KLMOptionsFrame_HealthLowLowSlider_OnValueChanged(self, value)
	if value >= tonumber(KLMOptionsFrame_HealthLowSlider:GetValue()) then
		value = tonumber(KLMOptionsFrame_HealthLowSlider:GetValue()) - 1
		KLMOptionsFrame_HealthLowLowSlider:SetValue(value)
	end
	KLMOptionsFrame_HealthLowLowSlider_Percent:SetText(tostring(floor(value)).." "..L["percent"])
end

function KLMOptionsFrame_PowerLowSlider_OnValueChanged(self, value)
	if value <= tonumber(KLMOptionsFrame_PowerLowLowSlider:GetValue()) then
		value = tonumber(KLMOptionsFrame_PowerLowLowSlider:GetValue()) + 1
		KLMOptionsFrame_PowerLowSlider:SetValue(value)
	end
	KLMOptionsFrame_PowerLowSlider_Percent:SetText(tostring(floor(value)).." "..L["percent"])
end

function KLMOptionsFrame_PowerLowLowSlider_OnValueChanged(self, value)
	if value >= tonumber(KLMOptionsFrame_PowerLowSlider:GetValue()) then
		value = tonumber(KLMOptionsFrame_PowerLowSlider:GetValue()) - 1
		KLMOptionsFrame_PowerLowLowSlider:SetValue(value)
	end
	KLMOptionsFrame_PowerLowLowSlider_Percent:SetText(tostring(floor(value)).." "..L["percent"])
end
